-- Puffin: A Tojaso FowlPlay Project
-- Model component for objectives tracker
-- Credit to Seerah for Who Framed Watcher Wabbit

local _, Puffin = ...
local EF = Puffin.Error
local DB = Puffin.Database

local Model_Objectives = {}

local OTF = ObjectiveTrackerFrame
local OTF_HeaderMenu = OTF.HeaderMenu
local OTF_MinimizeButton = OTF.HeaderMenu.MinimizeButton
local OTF_QuestHeader = ObjectiveTrackerBlocksFrame.QuestHeader
local _, OTF_QuestTitle = ObjectiveTrackerBlocksFrame.QuestHeader:GetRegions()
local OTF_AchievementHeader = ObjectiveTrackerBlocksFrame.AchievementHeader
local OTF_ScenarioHeader = ObjectiveTrackerBlocksFrame.ScenarioHeader
local OTF_BonusHeader = BONUS_OBJECTIVE_TRACKER_MODULE.Header
local OTF_Scenario = ScenarioStageBlock
local OTF_Container = CreateFrame("ScrollFrame", "OTF_Container", UIParent, "UIPanelScrollFrameTemplate")
local OTF_ContainerScrollBar = OTF_ContainerScrollBar
local OTF_Headers = {}
local OTF_Lines = {}
local media = {} -- set to defaults from database
local callback -- save callback registered for update

local function ExpandObjectiveTracker()
	if OTF.collapsed then
		OTF_Container:SetHeight(600)
		OTF_Container:SetWidth(265)
		OTF_Container:EnableMouseWheel(true)
		OTF_Container:SetVerticalScroll(0)
		ObjectiveTracker_MinimizeButton_OnClick()
	end
end

local function CollapseObjectiveTracker()
	if not OTF.collapsed then
		OTF_Container:SetHeight(30)
		OTF_Container:EnableMouseWheel(false)
		OTF_Container:SetVerticalScroll(0)
		ObjectiveTracker_MinimizeButton_OnClick()	--default function called by OnClick
	end
end

local function Objectives_Configure()
	OTF_Container:SetWidth(265)
	local OTFContainerScrollChild = CreateFrame("Frame", "OTFContainerScrollChild", OTF_Container)
	OTFContainerScrollChild:SetWidth(185)
	OTFContainerScrollChild:SetHeight(1000)
	OTF_Container:SetScrollChild(OTFContainerScrollChild)
	OTF_ContainerScrollBar.scrollStep = 35

	OTF:SetParent(OTFContainerScrollChild)
	OTF:SetClampedToScreen(false)
	OTF:ClearAllPoints()
	OTF:SetPoint("TOPLEFT", 30, -10)
	local cap = OTF.ClearAllPoints
	local sp = OTF.SetPoint
	hooksecurefunc(OTF, "SetPoint", function(self)
		cap(self)
		sp(self, "TOPLEFT", 30, -10)
		self:SetHeight(980)
	end)
	
	OTF_Container:ClearAllPoints()
	OTF_Container:SetPoint("TOPRIGHT", UIParent, "TOPRIGHT", 0, -20)

	OTF_Container:SetHeight(600)
	OTF_Container:SetScale(1)
	OTF:SetHeight(980)
	
	ObjectiveTrackerBlocksFrame.QuestHeader.Background:Hide()	
	ObjectiveTrackerBlocksFrame.AchievementHeader.Background:Hide()	
	ObjectiveTrackerBlocksFrame.ScenarioHeader.Background:Hide()	
	BONUS_OBJECTIVE_TRACKER_MODULE.Header.Background:Hide()	
	
	local font = media.font
	local fsize = media.fontSize
	local fflag = "None"
	local shadow = 1

	OTF_HeaderMenu.Title:SetAlpha(0)
	OTF_ContainerScrollBarScrollUpButton:SetAlpha(0)
	OTF_ContainerScrollBarThumbTexture:SetAlpha(0)
	OTF_ContainerScrollBar:SetAlpha(0)
	OTF_MinimizeButton:SetAlpha(0.25)

	OTF_QuestHeader.Text:SetFont(font, fsize+2, fflag)
	OTF_QuestHeader.Text:SetShadowColor(0, 0, 0, shadow)
	OTF_AchievementHeader.Text:SetFont(font, fsize+2, fflag)
	OTF_AchievementHeader.Text:SetShadowColor(0, 0, 0, shadow)
	OTF_ScenarioHeader.Text:SetFont(font, fsize+2, fflag)
	OTF_ScenarioHeader.Text:SetShadowColor(0, 0, 0, shadow)
	OTF_BonusHeader.Text:SetFont(font, fsize+2, fflag)
	OTF_BonusHeader.Text:SetShadowColor(0, 0, 0, shadow)
	OTF_Scenario.Stage:SetFont(font, fsize+5, fflag)
	
	for k,v in pairs(OTF_Headers) do
		v:SetFont(font, fsize, fflag)
		v:SetShadowColor(0, 0, 0, shadow)
	end
	for k,v in pairs(OTF_Lines) do
		v.Text:SetFont(font, fsize, fflag)
		v.Text:SetShadowColor(0, 0, 0, shadow)
		if v.Dash then
			v.Dash:SetFont(font, fsize, fflag)
			v.Dash:SetShadowColor(0, 0, 0, shadow)
		end
	end

	local function Objectives_Updates(module, block, objectiveKey, text, lineType, useFullHeight, hideDash, colorStyle)
		local header = block.HeaderText
		local line = block.lines[objectiveKey]
		if header and not header.isSkinned then
			header:SetFont(font, fsize, fflag)
			header:SetShadowColor(0, 0, 0, 1)
			tinsert(OTF_Headers, header)
			header.isSkinned = true
		end
		if line and not line.isSkinned then
			line.Text:SetFont(font, fsize, fflag)
			line.Text:SetShadowColor(0, 0, 0, 1)
			if line.Dash then
				line.Dash:SetFont(font, fsize, fflag)
				line.Dash:SetShadowColor(0, 0, 0, 1)
			end
			tinsert(OTF_Lines, line)
			line.isSkinned = true
		end
	end
	for i = 1, #OTF.MODULES do
		hooksecurefunc(OTF.MODULES[i], "AddObjective", Objectives_Updates)
	end
	hooksecurefunc(SCENARIO_TRACKER_MODULE, "AddObjective", Objectives_Updates)
	
	OTF_QuestHeader.animateReason = 0
	OTF_AchievementHeader.animateReason = 0
	
	OTF_MinimizeButton:Show()
	hooksecurefunc(OTF_MinimizeButton, "Hide", OTF_MinimizeButton.Show)
	
	ExpandObjectiveTracker()
	
	OTF_MinimizeButton:RegisterForClicks("AnyDown")
	OTF_MinimizeButton:SetScript("OnClick", function(self,btn)
		if btn == "LeftButton" then
			if not OTF.collapsed then	--if the container's not collapsed, collapse it
				CollapseObjectiveTracker()
			else
				ExpandObjectiveTracker()
			end
		elseif btn == "RightButton" then
			securecall("ToggleQuestLog")
		end
	end)
		
	OTF_MinimizeButton:SetScript("OnEnter", function(self)
			GameTooltip:SetOwner(self, "ANCHOR_TOPRIGHT")
			GameTooltip:AddLine("Objectives")
			GameTooltip:AddLine("|cffffff00Left-click|r to minimize/expand")
			GameTooltip:AddLine("|cffffff00Right-click|r to toggle quest log")
			GameTooltip:Show()
		end)
	OTF_MinimizeButton:SetScript("OnLeave", function() GameTooltip:Hide() end)
end

local function Objectives_OnEvent(_, event, ...)
	if event == "PLAYER_ENTERING_WORLD" then
		if IsInInstance() then CollapseObjectiveTracker() end -- collapse objectives when going into an instance
	elseif event == "QUEST_WATCH_LIST_CHANGED" or event == "TRACKED_ACHIEVEMENT_LIST_CHANGED" then
		if GetNumTrackedAchievements() == 0 and GetNumQuestWatches() == 0 and GetNumAutoQuestPopUps() == 0 then -- check if no objectives
			OTF_Container:SetAlpha(0)
			OTF_MinimizeButton:EnableMouse(false)
		else
			OTF_Container:SetAlpha(1)
			OTF_MinimizeButton:EnableMouse(true)
		end
	else
		local arg1 = ...
		if event == "QUEST_AUTOCOMPLETE" or arg1 == 0 then
			if not IsInInstance() then ExpandObjectiveTracker() end
		end
	end
end

function Model_Objectives.OnInitialize()
	FRAMELOCK_STATES.PETBATTLES.OTF_Container = "hidden"	
	OTF_Container:RegisterEvent("QUEST_AUTOCOMPLETE")
	OTF_Container:RegisterEvent("QUEST_WATCH_LIST_CHANGED")
	OTF_Container:RegisterEvent("TRACKED_ACHIEVEMENT_LIST_CHANGED")
	OTF_Container:SetScript("OnEvent", Objectives_OnEvent)		
	
	media.font = DB.GetAttribute("defaults", "media", "font")
	media.fontSize = DB.GetAttribute("defaults", "media", "fontSize")
	media.border = DB.GetAttribute("defaults", "media", "border")
	media.bar = DB.GetAttribute("defaults", "media", "bar")
	media.background = DB.GetAttribute("defaults", "media", "background")
	
	Objectives_Configure()
	
	-- also hide the order hall bar here...
	if OrderHallCommandBar then OrderHallCommandBar:SetScript("OnShow", hideEvent); OrderHallCommandBar:Hide() end
	UIParent:UnregisterEvent("UNIT_AURA") -- seems to be necessary so order hall bar stays hidden

	Puffin.UnregisterCallback(callback) -- only need to do this on first update
end
	
callback = Puffin.RegisterCallback("Puffin_Update", Model_Objectives.OnInitialize)
