-- Puffin: A Tojaso FowlPlay Project
-- Model component for automatically repairing gear and vendoring gray items

local _, Puffin = ...
local EF = Puffin.Error
local DB = Puffin.Database

local Model_Vendor = {}
local callback = false -- callback used to sell items, one item per timer event
local callbackActive = false
local itemCount, itemCost = 0, 0
local lastBag, lastSlot = 0, 1

-- Initialize by registering for relevant events
function Model_Vendor.OnInitialize()
	Puffin.RegisterCallback("MERCHANT_SHOW", Model_Vendor.Vendor_Show)
	Puffin.RegisterCallback("MERCHANT_CLOSED", Model_Vendor.Vendor_Hide)
	callback = Puffin.RegisterCallback("Puffin_Vendor_Sell", Model_Vendor.Vendor_Sell, 0, false)
end

-- Sell a gray item, if any
function Model_Vendor.Vendor_Sell()
	local found = false
	for bag = lastBag, 4 do -- check all bags
		for slot = lastSlot, GetContainerNumSlots(bag) do -- check all slots in the bag
			link = GetContainerItemLink(bag, slot)
			if link then
				local _, _, quality, _, _, _, _, _, _, _, vendorPrice = GetItemInfo(link)
				local _, count = GetContainerItemInfo(bag, slot)
				if quality == 0 and (vendorPrice or 0) > 0 then
					if MerchantFrame:IsShown() then -- make sure the vendor is still open for business
						UseContainerItem(bag, slot) -- this will sell the item
						itemCost = itemCost + (count * vendorPrice) -- update the running total
						itemCount = itemCount + 1 -- only add one for entire stack to keep it simple
						found = true -- made a sale on this pass
						lastBag = bag; lastSlot = slot + 1 -- remember so start search at same place
					end
				end
			end
		end
	end
	if found then
		Puffin.StartTimer("Puffin_Vendor_Sell", 0.1) -- restart timer for selling next item, limit to 10 sales per second
	else
		if itemCount > 0 and itemCost > 0 then -- all done, check if actually sold anything
			local t = string.format("Sold %d", itemCount)
			if itemCount == 1 then t = t .. " item for " else t = t .. " items for " end
			t = t .. GetCoinTextureString(itemCost)
			DEFAULT_CHAT_FRAME:AddMessage(t, 1, 1, 0)
			itemCount = 0; itemCost = 0
		end
	end
end

-- Start processing when vendor window is opened, then use timer to sell all items and do repairs
function Model_Vendor.Vendor_Show()
	if CanMerchantRepair() then -- first repair items if possible
		local repairAllCost, canRepair = GetRepairAllCost()
		if canRepair then
			if IsAltKeyDown() then RepairAllItems(1) end -- hold alt key when opening the vendor to try to use guild funds
			RepairAllItems()
			local t = "Repaired all items for " .. GetCoinTextureString(repairAllCost)
			DEFAULT_CHAT_FRAME:AddMessage(t, 1, 1, 0)
		end
	end
	itemCount = 0; itemCost = 0 -- running totals of vendored items
	lastBag = 0; lastSlot = 1 -- restart indices into the bags/slots
	if not callbackActive then Puffin.ResumeCallback(callback); callbackActive = true end
	Puffin.StartTimer("Puffin_Vendor_Sell", 0.1) -- start timer for selling items (ten sales per second)
end

-- Stop processing when vendor window is closed
function Model_Vendor.Vendor_Hide()
	Model_Vendor.Vendor_Sell() -- display results if vendor closed before completing sales
	if callbackActive then Puffin.SuspendCallback(callback); callbackActive = false end
end

Puffin.RegisterCallback("Puffin_Initialize", Model_Vendor.OnInitialize)
