-- Puffin: A Tojaso FowlPlay Project
-- Author: Tomber, contact tomber@toja.me
-- Copyright 2016, All rights reserved

local _, Puffin = ...
local EF = Puffin.Error
local DB = Puffin.Database

local Puffin_Test = {}
local throttleTime, updateCounter, refreshCounter = 0, 0, 0 -- counters for tracking update and refresh cycles
local timerCounter = 0 -- counter for tracking timer tests
local lastTime = 0 -- tracks time between refresh calls

function Puffin_Test.OnInitialize()
	-- Puffin.Debug("test", "OnInitialize")
	Puffin.RegisterCallback("Puffin_Test_Timer", Puffin_Test.Puffin_Test_Timer)
	lastTime = GetTime()
	if DB.IsDatabase("TestDB") then
		local value1 = DB.GetAttribute("TestDB", "test", "attribute1")
		local value2 = DB.GetAttribute("TestDB", "test", "attribute2")
		Puffin.Debug("restore_db", tostring(value1), tostring(value2))
	else
		DB.AllocateDatabase("TestDB", true)
		DB.SetAttribute("TestDB", "test", "attribute1", "value1")
		DB.SetAttribute("TestDB", "test", "attribute2", 2)
		local value1 = DB.GetAttribute("TestDB", "test", "attribute1")
		local value2 = DB.GetAttribute("TestDB", "test", "attribute2")
		Puffin.Debug("save_db", tostring(value1), tostring(value2))
	end
end

local panel_done = false
local text_widget
local bar_widget
local rect_widget

function Puffin_Test.OnUpdate()
	updateCounter = updateCounter + 1	
	Puffin.StartTimer("Puffin_Test_Timer", 0.05)

	if IsAltKeyDown() and Puffin.Class then
		if not panel_done then
			local panel1 = Puffin.Classes.Widget_Panel:new()
			Puffin.Debug("Panel1 = ", panel1)
			panel1:SetSize(400, 100)
			panel1:SetBackdrop("Interface\\AddOns\\Puffin\\Media\\TukuiBar", { r = 1, g = 0, b = 0, a = 1 }, 0, 2 )
			panel1:SetBorder("Interface\\AddOns\\Puffin\\Media\\BorderSquare2", { r = 0.1, g = 0.1, b = 0.1, a = 1 }, 8, 2)
			panel1:SetPosition("CENTER", nil, "CENTER")
			panel1:Update()

			local panel2 = Puffin.Classes.Widget_Panel:new()
			Puffin.Debug("Panel2 = ", panel2)
			panel2:SetSize(100, 200)
			panel2:SetBackdrop("Interface\\AddOns\\Puffin\\Media\\TukuiBar", { r = 0, g = 1, b = 0, a = 1 }, 0, 2 )
			panel2:SetBorder("Interface\\AddOns\\Puffin\\Media\\BorderSquare2", { r = 0.1, g = 0.1, b = 0.1, a = 1 }, 8, 2)
			panel2:SetPosition("CENTER", nil, "CENTER")
			panel2:Update()

			text_widget = Puffin.Classes.Widget_Text:new()
			Puffin.Debug("Text = ", text_widget)
			text_widget:SetSize(200, 20)
			text_widget:SetPosition("CENTER", nil, "CENTER", 400, 0)

			local template = { width = 40, height = 40 }
			local layout = { width = 60, height = 60, alpha = 0.5 }
			icon_widget = Puffin.Classes.Widget_Icon:new(template, layout)
			Puffin.Debug("Icon = ", icon_widget)
			-- icon_widget:SetSize(40, 40)
			icon_widget:SetPosition("CENTER", nil, "CENTER", -400, 0)
			icon_widget:SetTexture([[Interface\CharacterFrame\UI-StateIcon]], { .5, 1, 0, .49 })
			icon_widget:SetColor({ r = 1, g = 0, b = 0, a = 1 })
			icon_widget:Update()
			
			bar_widget = Puffin.Classes.Widget_Bar:new()
			Puffin.Debug("Bar = ", bar_widget)
			bar_widget:SetSize(210, 24)
			bar_widget:SetPosition("CENTER", nil, "CENTER", 0, 300)
			bar_widget:SetBars("Interface\\AddOns\\Raven\\Statusbars\\Gray", { r = 1, g = 0, b = 0, a = 1 }, 1,
				"Interface\\AddOns\\Raven\\Statusbars\\Gray", { r = 1, g = 0, b = 0, a = 1 }, 0.5 )
			-- bar_widget:SetBackdrop([[Interface\BUTTONS\WHITE8X8.blp]], { r = 1, g = 0, b = 0, a = 0 }, 0, 1)
			bar_widget:SetBorder([[Interface\BUTTONS\WHITE8X8.blp]], { r = 0, g = 0, b = 0, a = 1 }, 1, 1)
			-- bar_widget:SetValue(0.5)
			bar_widget:SetTime(GetTime() + 20, 20)
			bar_widget:Set("sparkAlpha", 0)
			bar_widget:Update()

			bar2_widget = Puffin.Classes.Widget_Bar:new()
			Puffin.Debug("Bar = ", bar2_widget)
			bar2_widget:SetSize(210, 24)
			bar2_widget:SetPosition("CENTER", nil, "CENTER", 0, 340)
			bar2_widget:SetBars("Interface\\AddOns\\Raven\\Statusbars\\Gray", { r = 1, g = 0, b = 1, a = 1 }, 1,
				"Interface\\AddOns\\Raven\\Statusbars\\Gray", { r = 1, g = 0, b = 1, a = 1 }, 0.5 )
			-- bar2_widget:SetBackdrop([[Interface\BUTTONS\WHITE8X8.blp]], { r = 1, g = 0, b = 0, a = 0 }, 0, 1)
			bar2_widget:SetBorder([[Interface\BUTTONS\WHITE8X8.blp]], { r = 0, g = 0, b = 0, a = 1 }, 1, 1)
			-- bar2_widget:SetValue(0.5)
			bar2_widget:SetTime(GetTime() + 20, 20)
			bar2_widget:Set("sparkAlpha", 0)
			bar2_widget:Update()
			
			rect_widget = Puffin.Classes.Widget_Rectangle:new()
			Puffin.Debug("Rect = ", rect_widget)
			rect_widget:SetSize(100, 100)
			rect_widget:SetPosition("CENTER", nil, "CENTER", 0, 200)
			rect_widget:SetBorder("Interface\\AddOns\\Puffin\\Media\\BorderSquare1", { r = 0.1, g = 0.1, b = 0.1, a = 1 }, 8, 2)
			rect_widget:SetRectangle("Interface\\AddOns\\Puffin\\Media\\TukuiBar", { 0, 1, 0, 1 }, { r = 0, g = 0, b = 1, a = 1 })
			rect_widget:Set("valueWidth", 0.75)
			rect_widget:Set("flashPeriod", 2)
			rect_widget:Update()
			
			panel_done = true
		end
		text_widget:SetText(tostring(GetTime()))
		text_widget:SetColor({ r = 0, g = 1, b = 0, a = 1 })
		text_widget:Update()
	end
end

function Puffin_Test.OnRefresh()
	refreshCounter = refreshCounter + 1	
	local now = GetTime(); local elapsed = now - lastTime; lastTime = now
	throttleTime = throttleTime + elapsed -- timer to make sure message happen once per second
	if throttleTime >= 1.0 then
		if bar_widget then bar_widget:Update() end
		if bar2_widget then bar2_widget:Update() end
		-- Puffin.Debug("update", updateCounter, "refresh", refreshCounter, "timer", timerCounter)
		throttleTime = 0; updateCounter = 0; refreshCounter = 0; timerCounter = 0
	end
end

function Puffin_Test.Puffin_Test_Timer()
	timerCounter = timerCounter + 1
	-- Puffin.Debug("Timer!", timerCounter)
end

Puffin.RegisterCallback("Puffin_Initialize", Puffin_Test.OnInitialize)
Puffin.RegisterCallback("Puffin_Update", Puffin_Test.OnUpdate)
Puffin.RegisterCallback("Puffin_Refresh", Puffin_Test.OnRefresh)
